//--------------------------------------------------------------------------------------
/**
 * <p>Title: ClientStockData </p>
 * <b> Category: </b> Baseline
 * <b> Behavior: </b> This object stores a client's stock data. It is set up so that
 *     the client data is easily accessible to a JTable object.
 * <b> Inherits from: </b> javax.swing.table.AbstractTableModel
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------

package powerquotes.client;

// Import
import javax.swing.table.*;
import java.util.*;
import powerquotes.common.*;
import org.apache.log4j.Logger;

/** 
 * Stores a client's stock data.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
public class ClientStockData extends AbstractTableModel
{

    // all stocks available to look at
    private Vector allStocks = null;

    // stocks the user is interested in
    private HashMap stocks = null;
    private Vector ids = null;
    private String[] columnNames = {"Symbol", "Name", "Price", "Last Updated"};
        
    protected static Logger log = Logger.getLogger("powerquotes.client.ClientStockData");;


    /**
     * Default constructor.
     */
    public ClientStockData()
    {
        stocks = new HashMap();
        ids = new Vector();
        allStocks = new Vector();
    }
 
    /**
     * This method will add or replace the info for the given stock.
     * @param stock stock info to add/replace the client data
     */
    public void setStock(StockInfo stock)
    {
        // if stock is null - do nothing
        if(stock == null)
        {
            log.debug("stock given is null");
            return;
        }   
    
        // add/replace stock
        StockInfo oldInfo = (StockInfo) stocks.put(new Integer(stock.getId()), stock);

        // if new stock, add its id to the ids vector
        if(! ids.contains(new Integer(stock.getId())) )
        {
            ids.add(new Integer(stock.getId()));
            log.debug("IDs updated to contain: " + stock.getId());
            log.debug("IDs size: " + ids.size());
        }

        log.debug("Updated Cache with: " + stock);
        log.debug("Old info was: " + oldInfo);
    }


    /**
     * This method will add or replace the info with the given stocks.
     * @param stocks array of stocks to add/replace the client data
     */
    public void setStocks(Vector stocks)
    {
        // if vector is null - do nothing
        if(stocks == null)
        {
            log.debug("stock vector is null");
            return;
        }
        
        for(int i=0; i<stocks.size(); i++)
        {
            setStock((StockInfo) stocks.elementAt(i));
        }
    }


    /**
     * This method will remove a stock from the client data.
     * @param stock stock to remove from client data
     */
    public void removeStock(StockInfo stock)
    {
        // if stock is null - do nothing
        if(stocks == null)
        {
            log.debug("stock given is null");
            return;
        }
        
        // will remove the stock if it exists or do nothing if it doesn't
        // should remove element from both or neither - a stock id should never be
        // in one and not the other.  <-- is there a way to enforce this
        ids.removeElement(new Integer(stock.getId()));
        stocks.remove(new Integer(stock.getId()));
    }

    /**
    * This method will remove the stocks from the client data
    * @param stocks vector of stocks to remove from the client data
    */
    public void removeStocks(Vector stocks)    
    {
        // if vector is null - do nothing
        if(stocks == null)
        {
            log.debug("stock vector is null");
            return;
        }
        
        for(int i=0; i<stocks.size(); i++)
        {
            removeStock((StockInfo) stocks.elementAt(i));
        }
    }

    /**
     * Sets all of the stocks available to the client.
     *@param stocks vector of all stocks available to the client 
     */
    public void setAllStocks(Vector stocks)
    {
        // if vector is null - do nothing
        if(stocks == null)
        {
            log.debug("stock vector is null");
            return;
        }
        
        allStocks = stocks;
    }


    /**
     * Gets all of the stocks available to the client.
     * @return a vector of all stocks available to the client
     */
    public Vector getAllStocks()
    {
        // make sure that null is never returned
        if(allStocks != null)
            return(allStocks);
        else
            return(new Vector());
    }

    
    /**
     * Gets all of the stocks that the client is currently interested in
     * @return a vector of stocks
     */
    public Vector getClientStocks()
    {
        // make sure that null is never returned
        if(allStocks != null)
            return(new Vector(stocks.values()));
        else
            return(new Vector());
    }


    //================================================================================
    // AbstractTableModel Methods
    //================================================================================
    /**
     * This method returns the number of stocks that are displayed in the table.
     *
     * @return the number of rows in the table.
     */
    public int getRowCount()
    {
        return stocks.size();
    }


    /**
     * This method returns the number of columns in the table.
     *
     * @return the number of columns in the table
     */
    public int getColumnCount()
    {
        return columnNames.length;
    }


    /**
     * This method returns the object at the given indices in the table.
     *
     * @param r row number
     * @param c column number
     * @return object located at r,c in the table or null if index is out of range
     */
    public Object getValueAt(int r, int c)
    {
        // make sure index is valid
        if(r < ids.size())
        {
        // get stock id at given index
            Integer stockId = (Integer) ids.elementAt(r);

            // get stock from hash table
            StockInfo stock = (StockInfo) stocks.get(stockId);

            // get "column" information from stock
            String str;
            switch(c)
            {
                case 0:
                    str = stock.getSymbol();
                    break;
                case 1:
                    str = stock.getName();
                    break;
                case 2:
                    str = String.valueOf(stock.getPrice());
                    break;
                case 3:
                    Date date = new Date(stock.getTimestamp());
                    str = date.toString();
                    break;
                default:
                    str = "";
            }

            return(str);
        }

        return null;
    }


    /**
     * This method returns the name of the given column.
     *
     * @param c column number
     * @return string name for column c
     */
    public String getColumnName(int c)
    {
        // make sure that a valid string is always returned
        if(c < columnNames.length)
            return (columnNames[c]);

        else
            return ("");
    }

}
