//--------------------------------------------------------------------------------------
/**
 * <p>Title: PowerQuotesClient </p>
 * <b> Category: </b>  Baseline
 * <b> Behavior: </b> Provides a client for the system.
 * <b> Inherits from: </b>  java.lang.Thread
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------
package powerquotes.client;

// Import
import javax.rmi.*;
import java.io.*;
import java.util.*;
import powerquotes.ejb.*;
import powerquotes.common.*;
import org.apache.log4j.Logger;

import java.rmi.RemoteException;

/**
 * Provides a client for the system.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
public class PowerQuotesClient extends Thread 
{
    // variables
    private String userName = null;               // user name
    private UserProfile userProfile = null;       // user profile

    UPSession upSessionBean = null;               // profile session bean instance
    ClientUpdate clientUpdateBean = null;         // client update session bean instance

    private ClientStockData stockData = null;     // client's stock data
    private static PowerQuotesFrame frame = null; // GUI

    // for threading
    public static final long sleepTime = 1000;
    public static boolean updateNeeded= false;

    // server numbers
    private int serverCount = 1;
    private int currentServer = 0;
    Random randomGen = null;

    protected static Logger log = Logger.getLogger("powerquotes.client.PowerQuotesClient");


    /**
     * Constructor with server count.
     *
     * @category Baseline
     * @param sCount total number of servers
     */
    public PowerQuotesClient(int sCount)
    {
        // set variables
        serverCount = sCount;
        currentServer = getNextServerNum();

        initClient();
    }

    /**
     * Constructor with server count and server number.
     *
     * @category Baseline
     * @param sCount total number of servers
     * @param serverNum server number that client should connect to initially.
     */
    public PowerQuotesClient(int sCount, int serverNum)
    {
        // set the server variables
        serverCount = sCount;
        currentServer = serverNum;

        initClient();
    }

    /**
     * Initializes the client and GUI (if a GUI exists)
     */
    private void initClient()
    {
        // print status
	log.debug("init client");

        // create ClientStockData - allstocks set in connectToServer
        stockData = new ClientStockData();

        // connect to the current server
        connectToServer(currentServer);

        // initialize the GUI
        if(this.frame != null)
        {
            // set the GUI's client
            frame.setClient(this);

            // prompt for user name & login user
            setUserName(this.frame.getUserName());
            serverLogin();

            // subscribe user to stocks
            serverSubscribe();
            stockData.setStocks(serverGetUpdatedStocks());
            frame.updateStockNames();

            // start the thread to update the stock data on a regular basis
            start();
        }
    }


    /**
     * This method runs the thread.  The thread checks for updated stocks every so often.
     * @category Baseline
     */
    public void run()
    {
        while(!interrupted())
        {
            try
            {
                // thread has been flagged for an update
                // the flag is set externally when the profile has been changed
                if(this.updateNeeded == true)
                {
                    // print status
                    log.debug("performing update");

                    // set the user profile
                    serverSetUserProfile();

                    // subscribe user to stocks
                    serverSubscribe();
                    stockData.setStocks(serverGetUpdatedStocks());

                    // update the stock names
                    if(this.frame !=null)
                    {
                        this.frame.updateStockNames();
                        this.frame.updateStocks();
                    }

                    this.updateNeeded = false;
                }
                else
                {
                    // print status
                    log.debug("update");

                    // call to get updated stocks & update client data
                    stockData.setStocks(serverGetUpdatedStocks());

                    // update gui
                    if(this.frame != null)
                    {
                        this.frame.updateStocks();
                    }
                }

                // wait
                sleep(this.sleepTime);
            }
            // if another thread has interrupted this current thread. - do we want to exit??
            catch(InterruptedException ie)
            {
                log.fatal("Caught an interruped exception! " + ie.toString(), ie);
                System.exit(0);
            }
            // some other problem - do we want to exit???
            catch(Exception e)
            {
                log.fatal("Caught an unexpected exception! " + e.toString(), e);
                System.exit(0);
            }
        }
    }


    /**
     * Gets the next server number.
     * @category Fault Tolerance
     *
     * @return the next server number that is not equal to the current server number
     */
    private int getNextServerNum()
    {
        // if currentServer = 0 - this is the first time and no
        // server number was given so pick a random server number
        if(currentServer < 1)
        {
            // if only 1 server, always use that one
            if(serverCount ==1)
                currentServer = 1;
                
            //else get a random number
            else
            {
                // create the random number generator (if it is not already there)
                if(randomGen == null)
                    randomGen = new Random();

                // get a random number
                currentServer = randomGen.nextInt(serverCount) + 1;
                log.debug("generated random number " + currentServer);
            }     
        }
        
        // if not first server, get the next server number
        else
        {
            currentServer = (currentServer %  serverCount) + 1;
        }       
    
        return(currentServer);
    }


    //================================================================================
    // Server Functions
    //================================================================================
    /**
     * Connects to a server.
     *
     * @category Baseline, Fault Tolerance
     * @param num server number to connect to
     */
    private void connectToServer(int num)
    {
        // bind to the beans
        try
        {
            log.info("trying to connect to server " + num);

            // session beans
            upSessionBean = Utilities.bindToUserProfileSession(num);
            clientUpdateBean = Utilities.bindToClientUpdateSession(num);

            // make sure that the available stocks matches the server stocks
            stockData.setAllStocks(clientUpdateBean.getAllStocks());

            // print out connection time
            Date date = new Date(System.currentTimeMillis());
            log.info("connected to server " + currentServer + " at " + date.toString());
            
            // set current server
            currentServer = num;            
        }
        // if problem connecting, just try another one
        // This appears to not be thrown anywhere right now    (** actually it is thrown in Utilities **)
        catch(StockServerException sse)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a stock server exception during connectToServer! " + sse.toString(), sse);
            connectToServer(getNextServerNum());
        }
        catch(RemoteException re)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a remote exception during connectToServer! " + re.toString(), re);
            connectToServer(getNextServerNum());
        }
        // some other problem - do we want to exit???
        catch(Exception e)
        {
            log.fatal("Caught an unexpected exception during connectToServer! " + e.toString(), e);
            System.exit(0);
        }
    }

    /**
     * Logs in the current user.
     * @category Baseline, Fault Tolerance
     */
    public void serverLogin()
    {
        // if profile exists, get profile
        // if not (create a new one)
        UserProfile up = null;
        try
        {
            up = upSessionBean.getUserProfile(userName);
            setUserProfile(up);
	    log.debug("got user profile");
        }
        catch(NoSuchUserException nsue)
        {
            // user did not exist, so create a new profile
            up = new UserProfile();
            up.setUserName(userName);

            // set the user profile to the new one
            setUserProfile(up);
            serverSetUserProfile();
            log.debug("created new user profile");
        }
        // if exception occurs, connect to new server and try to login again
        catch(StockServerException sse)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a stock server exception during serverLogin! " + sse.toString(), sse);
            connectToServer(getNextServerNum());
            serverLogin();
        }
        catch(NotSerializableException nse)       // what to do here ???
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a not serializable exception during serverLogin! " + nse.toString(), nse);
            connectToServer(getNextServerNum());
            serverLogin();
        }
        catch(RemoteException re)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a remote exception during serverLogin! " + re.toString(), re);
            connectToServer(getNextServerNum());
            serverLogin();
        }
        // some other problem - do we want to exit???
        catch(Exception e)
        {
            log.fatal("Caught an unexpected exception during serverLogin! " + e.toString(), e);
            System.exit(0);
        }
    }

    /**
     * Sets the database profile to the current profile.
     * @category Baseline, Fault Tolerance
     */
    public void serverSetUserProfile()
    {
        try
        {
            // set the profile
            upSessionBean.setUserProfile(userProfile);
	    log.debug("set user profile");
        }
        // if exception occurs, connect to new server and try to set profile again
        catch(StockServerException sse)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a stock server exception during serverSetUserProfile! " + sse.toString(), sse);
            connectToServer(getNextServerNum());
            serverSetUserProfile();
        }
        catch(RemoteException re)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a remote exception during serverSetUserProfile! " + re.toString(), re);
            connectToServer(getNextServerNum());
            serverSetUserProfile();
        }
        // some other problem - do we want to exit???
        catch(Exception e)
        {
            log.fatal("Caught an unexpected exceptiond during serverSetUserProfile! " + e.toString(), e);
            System.exit(0);
        }
    }


    /**
     * Subscribes the current user to the current profile's stock list.
     * @category Baseline, Fault Tolerance
     */
    public void serverSubscribe()
    {
        try
        {
            clientUpdateBean.subscribe(userName, userProfile.getStockList());
            log.debug("subscribed user");
        }
        // if exception occurs, connect to new server and try to subscribe again
        catch(ServerNotReadyException se)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a server not ready exception during serverSubscribe! " + se.toString(), se);
            connectToServer(getNextServerNum());
            serverSubscribe();
        }
        catch(RemoteException re)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a remote exception during serverSubscribe! " + re.toString(), re);
            connectToServer(getNextServerNum());
            serverSubscribe();
        }
        // some other problem - do we want to exit???
        catch(Exception e)
        {
            log.fatal("Caught an unexpected exception during serverSubscribe! " + e.toString(), e);
            System.exit(0);
        }
    }


    /**
     * Gets the updated stocks.
     * @category Baseline, Fault Tolerance
     */
    private Vector serverGetUpdatedStocks()
    {
        Vector stocks = null;
        try
        {
            // get the stock data
	    stocks = clientUpdateBean.getUpdatedStocks(userName);
        }
        // if exception occurs, connect to new server, subscribe client and try to get stocks again
        // if exception occurs, connect to new server and try to subscribe again
        catch(ServerNotReadyException se)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a server not ready exception during serverGetUpdatedStocks!" + se.toString(), se);
            connectToServer(getNextServerNum());
            serverSubscribe();
            stocks = serverGetUpdatedStocks();
        }
        catch(RemoteException re)
        {
            Date date = new Date(System.currentTimeMillis());
            log.info("Detected server exception at: " + date.toString());
            log.error("Caught a remote exception during serverGetUpdatedStocks! " + re.toString(), re);
            connectToServer(getNextServerNum());
            serverSubscribe();
            stocks = serverGetUpdatedStocks();
        }
        // some other problem - do we want to exit???
        catch(Exception e)
        {
            log.fatal("Caught an unexpected exception during serverGetUpdatedStocks! " + e.toString(), e);
            System.exit(0);
        }

        return(stocks);
    }

    //================================================================================
    // Accessor Functions
    //================================================================================
    /** Sets the userName.  This does not login to the server.  To do this, set the
     *  user name and then call login().
     * @category Baseline
     * @param name user name associated with the client
     */
    public void setUserName(String name)
    {
        userName = name;

        // if gui, change the name
        if(frame != null)
            frame.setTitle(frame.getTitle() + " - " + name);
    }

    /** Gets the userName.
     * @category Baseline
     * @return the user name associated with the client
     */
    public String getUserName() { return(this.userName); }


    /** Sets the userProfile.  This does not update the database profile.  To do this,
     *  set the user profile andthen call setServerUserProfile().
     * @category Baseline
     * @param profile user profile associated with the client
     */
    public void setUserProfile(UserProfile profile) { this.userProfile = profile; }

    /** Gets the userProfile.
     * @category Baseline
     * @return user profile associated with the client
     */
    public UserProfile getUserProfile() { return(this.userProfile); }


    /** Sets the stockData.
     * @category Baseline
     * @param data the client's stock data
     */
     public void setStockData(ClientStockData data) { this.stockData = data; }

    /** Gets the stockData.
     * @category Baseline
     * @return the client's stock data
     */
    public ClientStockData getStockData() { return(this.stockData); }


    //================================================================================
    // Main
    //================================================================================
    /**
    * This method checks for the correct arguments and creates a client.
    *
    * @category Baseline
    * @param args args[0] should be the total number of servers and optionally args[1]
    *        should be the initial server number.
    */
    public static void main(String[] args)
    {
        org.apache.log4j.xml.DOMConfigurator.configure("log4j.xml");

        // make sure number of agruments is correct
        if(args.length < 1)
        {
            printUsage();
            System.exit(0);
        }

        // the GUI is created here so that any tests do not have to use the GUI
        frame = new PowerQuotesFrame();

        // if one arguement - then no initial server specified
        PowerQuotesClient client = null;
        try
        {
            if(args.length == 1)
                client = new PowerQuotesClient(Integer.parseInt(args[0]));
            else
                client = new PowerQuotesClient(Integer.parseInt(args[0]), Integer.parseInt(args[1]));
        }
        catch(Exception e)
        {
            log.error(e.toString(), e);
            printUsage();
        }
    }


    /**
     * @category Baseline
     * Prints the usage string and exits
     */
    private static void printUsage()
    {
        log.fatal( "Usage: java powerquotes.client.PowerQuotesClient serverCount [initialServer]..." );
        log.fatal( "  serverCount: the total number of servers available" );
        log.fatal( "  initialServer: the initial server number of the client" );
    }


}


