//--------------------------------------------------------------------------------------
/**
 * <p>Title: PowerQuotesFrame </p>
 * <b> Category: </b> Baseline 
 * <b> Behavior: </b> * Provides the main GUI for the application.
 * <b> Inherits from: </b> javax.swing.JFrame 
 * <b> Clients: </b> N/A
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//--------------------------------------------------------------------------------------
package powerquotes.client;

// Import
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.table.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import powerquotes.client.*;
import powerquotes.common.*;


/**
 * Provides the main GUI for the application. 
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0 
 */
public class PowerQuotesFrame extends JFrame implements ActionListener
{

    // GUI variables
    private JScrollPane scrollPane = null;
    private JTable table = null;
    private JTabbedPane tabbedPane = null;
    private StockListDisplay addDisplay = null;
    private StockListDisplay removeDisplay = null;

    // for frame size
    private static final int inset = 150;

    // client
    private PowerQuotesClient client = null;


    /**
     * Default constructor.
     */
    public PowerQuotesFrame()
    {
        this.initComponents();
    }


    /**
     * This method initializes the GUI components.
     */
    private void initComponents()
    {
        // create components
        table = new JTable();
        scrollPane = new JScrollPane(table);
        tabbedPane = new JTabbedPane();

        // create add/remove stock panels (this should listen)
        addDisplay = new StockListDisplay("Add Stocks", this);
        removeDisplay = new StockListDisplay("Remove Stocks", this);

        // set component properties
        scrollPane.getViewport().setBackground(Color.white);
        tabbedPane.setPreferredSize(new Dimension(200, 50));

        // frame properties
        setTitle("PowerQuotes");
        addWindowListener(new WindowAdapter()
        {
            public void windowClosing(WindowEvent evt)
            {
                exitForm(evt);
            }
        });

        // tabbed pane
        tabbedPane.addTab("Add", addDisplay);
        tabbedPane.setSelectedIndex(0);
        tabbedPane.addTab("Remove", removeDisplay);

        // set frame position
        int width = Toolkit.getDefaultToolkit().getScreenSize().width - inset*2;
        int height = Toolkit.getDefaultToolkit().getScreenSize().height- inset*2;
        setBounds(inset, inset, width, height);

        // add components to frame
        getContentPane().add(tabbedPane, BorderLayout.WEST);
        getContentPane().add(scrollPane, BorderLayout.CENTER);

        // show GUI
        show();
    }


    /**
     * Exit the Application
     */
    private void exitForm(WindowEvent evt)
    {
        System.exit(0);
    }



    //================================================================================
    // GUI Listeners
    //================================================================================
    /**
     * Listens to action events.
     *
     * @param e action event to handle.
     */
    public void actionPerformed(ActionEvent e)
    {
        Vector selected = null;
        UserProfile up = null;

        // for add panel
        if(e.getActionCommand() == this.addDisplay.getPanelName())
        {
            // get selected stocks
            selected = this.addDisplay.getSelectedStocks();

            // update the profile
            up = this.client.getUserProfile();
            up.addStocks(selected);
            this.client.setUserProfile(up);
        }

        // for remove panel
        else if (e.getActionCommand() == this.removeDisplay.getPanelName())
        {
            // get selected stocks
            selected = this.removeDisplay.getSelectedStocks();

            // update the profile
            up = this.client.getUserProfile();
            up.removeStocks(selected);
            this.client.setUserProfile(up);

            // remove from stock data
            ClientStockData stockData = this.client.getStockData();
            stockData.removeStocks(selected);
            this.client.setStockData(stockData);
        }

        // flag for update
        this.client.updateNeeded = true;
      }


    //================================================================================
    // Accessor Functions
    //================================================================================
    /**
     * Sets the client.
     * @param c client to associate with the GUI
     */
    public void setClient(PowerQuotesClient c)
    {
        // set the local variable
        this.client = c;
        this.table.setModel(c.getStockData());

        // set the stock data based on the current client

        this.updateStockNames();
    }

    /**
     * Gets the client.
     * @return client associated with the GUI
     */
    public PowerQuotesClient getClient() { return(this.client);}


    //================================================================================
    // Client Interface Methods
    //================================================================================
    /**
     * This function will update the stock names for the add/remove list boxes. This
     * needs to be done every time the client modifies his/her profile.
     */
    public void updateStockNames()
    {
        if(this.client != null)
        {
            // get all stock names
            Vector allStocks = client.getStockData().getAllStocks();
            this.addDisplay.setStocks(allStocks);

            // get client stock names
            Vector clientStocks = client.getStockData().getClientStocks();
            this.removeDisplay.setStocks(clientStocks);
        }
    }


    /**
     * This method revalidates and repaints the stock table.
     */
    public void updateStocks()
    {
        table.revalidate();
        table.repaint();
    }


	/**
     * This method displays a message to the screen.
     * @param msg message to display to screen
     */
    public void displayMessage(String msg)
    {
        if(client != null)
            JOptionPane.showMessageDialog(this, msg);
    }


    /**
     * This method prompts for a user name.
     *@return the user name that is input by the user
     */
    public String getUserName()
    {
        String result = "";

        if(client != null)
        {
            result = JOptionPane.showInputDialog(this, "Please input your user name: ");

            if(result == null)
            {
                exitForm(null);
            }
        }

        return result;              //****** make sure valid name??
    }

}
