/**
 * <p>Title: StockCache </p>
 * <b> Category:  </b> Baseline
 * <b> Behavior: </b> Local cache of stock information
 * <b> Inherits from: </b> none
 * <b> Clients: </b> TickReceiverSessionBean, StockInterfaceSessionBean
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 *
 */
package powerquotes.common;

// Import
import powerquotes.ejb.StateReplicator;
import java.util.*;
import java.io.Serializable;
import java.rmi.RemoteException;

import org.apache.log4j.Logger;


/**
  * Singleton class that represents the system's current knowledge about stocks
  * This is updated as updates come in, and accessible when to service client subscriptions
  * 
  * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
  * @version 1.0 
  */
public class StockCache implements Serializable {

    private transient static StockCache theStockCache = null;
    private HashMap cache = null;

    // whether or not the cache has been initialized
    protected transient boolean initialized;
    
    protected transient boolean processQueuedUpdates;
    
    // queue for stuff we get before we've gotten an initial state
    // add to the end, take from the beginning
    private transient LinkedList backlog = null;

    // we should A) get this from a config file, B) use a real logger
    //    private static final boolean debug = true;
    //using real logger
    protected static transient Logger log = Logger.getLogger("powerquotes.common.StockCache");


    /***************************   Public Methods    **********************************/

    /**
     * @return the instance of the StockCache
     */
    public static StockCache getInstance() {

        if( theStockCache == null ) {
            theStockCache = new StockCache();
        }
        return theStockCache;
    }

    
    /**
     * @param stock stock update to process
     */
    public synchronized void setStock( StockInfo stock ) {
        if ( initialized ) {
            updateStock( stock );
        }
        else {
            // it's a queue - add to the end, take from the beginning
            backlog.addLast( stock );
        }
    }


    /*
     * @return a collection of all stocks in cache
     */
    public synchronized Collection getAllStocks() throws ServerNotReadyException {
          if( initialized ) {
              return cache.values();
          }
          else { 
              throw new ServerNotReadyException();
          }
    }

    /**
     *@ return a hashmap of the internal cache
     */
    public synchronized HashMap getInternalCache() {
        return cache;
    }

    /**
      * @return the StockInfo for this stockId or null if it wasn't found
      */
    public synchronized StockInfo getStock( Integer stockId ) throws ServerNotReadyException {
        if( initialized ) {
            return (StockInfo)cache.get( stockId );
        }
        else { 
            throw new ServerNotReadyException();
        }
    }
    
    /** 
     * @return whether the cache has been initialized or not
     */
    public boolean isInitialized() {
        return initialized;
    }


    /** 
     * Initialize the cache with the state from a replica
     */
    public void initializeCache( StockCache otherCache ) {
        
        log.info( "Trying to Initialize stock cache from other server... " );

        //StateReplicator stateReplicator = findOtherServer();
        
        // StockCache otherCache = null;
        // we found another server.... 
        /*
          if( stateReplicator != null ) {
          
          try {
          otherCache = stateReplicator.getStockCache();
          } catch( RemoteException re ) {
          log.error( "Could not get cache from other server: " + re );
          log.error( "Maybe it just went down? Let's find another..."  );
          // what do we do here? this *might* mean the server just went down...
          // ... in that case we should try again and get another server
          // if the server is down we won't be able to bind to it and it 
          // should just get skipper over next time
          initializeCache();
          }
        }
        */
        // if no servers - cache will be null - nothing to be done
        // This is normal if we're the first server to come up.
        if( otherCache != null ) {
            this.cache = otherCache.getInternalCache();
            log.debug( "other cache that I received: " + otherCache );
        }

        // process whatever stock updates may have queued up.
        processQueuedUpdates();

        // lastly mark the StockCache as initialized for normal updates
        initialized = true;
    }


    /***************************   Private Methods    **********************************/

    /**
     * Private constructor
     */
    private StockCache() {
        cache = new HashMap();
        backlog = new LinkedList();


        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        // Change this to false when the initializer is done
        // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        initialized = false;        
    }


    private synchronized void processQueuedUpdates() {

        log.info( "---- Beginning processing backlogged stock updates" );
        Iterator it = backlog.iterator();
        while( backlog.size() != 0 ) {
            StockInfo info = (StockInfo) backlog.removeFirst();
            log.info( "      Applying queued update: " + info );
            updateStock( info );
        }
        log.info( "---- Finished processing backlogged stock updates" );
    }



    private void updateStock( StockInfo stock ) {
        StockInfo oldInfo = (StockInfo) cache.put( new Integer( stock.getId() ), stock );
        log.debug( "NEW: Updated Cache with: " + stock );
        log.debug( "NEW: Old info was:" + oldInfo );
    }

    
    private StateReplicator findOtherServer() {

        log.info( "Looking for other servers: " );

        StateReplicator stateReplicator = null;
        int serverId = Integer.parseInt( System.getProperty( "powerquotes.serverId" ) );
        int numServers = Integer.parseInt( System.getProperty( "powerquotes.numServers" ) );

        log.info( "I am server " + serverId + " of " + numServers );
        
        for( int i = 1; i <= numServers; i++ ) {
            if( i == serverId ) {
                continue;
            }
            log.info( "  Trying server " + i + " ...");
            try {
                stateReplicator = Utilities.bindToStateReplicator( i );
            } catch( Exception e ) {   // we don't care why it fails.... 
                log.warn( "  Cannot connect to server " + i);
                continue;
            }
            log.info( "  Made connection to server " + i );

            // right now we always harass the first one on the list 
            // something more egalitarian might be better
            break;
        }
        return stateReplicator;
    }


    public String toString() {

        
        StringBuffer buffer = new StringBuffer();

        Collection c = cache.values();
        buffer.append( "Cache size is: " + c.size() );
        Iterator it = c.iterator();
        while( it.hasNext() ) {
            buffer.append( it.next() + "\n" );
        }
        return buffer.toString();
    }
}
