/**
 * <p>Title: StockTicker </p>
 * <b> Category: Baseline </b>
 * <b> Behavior: </b> Provides a simulation of a stock feed to the system.
 * <b> Inherits from: N/A </b>
 * <b> Clients: </b> none
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
package powerquotes.datafeed;

// imports
import powerquotes.common.StockInfo;
import powerquotes.ejb.TickReceiver;
import powerquotes.ejb.TickReceiverHome;
import powerquotes.ejb.impl.TickReceiverUtil;
import powerquotes.ejb.impl.TickReceiverSessionBean;

import javax.naming.NamingException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.ejb.CreateException;
import javax.rmi.PortableRemoteObject;

import java.rmi.RemoteException;
import java.util.*;
import java.io.*;
import java.math.BigDecimal;

import org.apache.log4j.Logger;


/**
  * This class provides a simulated data feed for stock updates.
  *
  * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
  * @version 1.0
  */
public class StockTicker {

    private static final String DELIM = ":";
    private String[] servers = null;              // array of object home names
    private TickReceiver[] tickReceivers = null;  // array of tick receivers
    private Properties stockProps = null;

    // list of stocks - index of the Vector is NOT the stock id
    private Vector stockList = null;

    protected static Logger log;


    /**
    * This method checks for the correct arguments and creates a stock ticker.
    *
    * @category Baseline
    * @param args list of object home names to bind to, updates per burst,
    */
    public static void main( String args[] )
    {
        //initialize log4j
        org.apache.log4j.xml.DOMConfigurator.configure("log4j.xml");
        log = Logger.getLogger("powerquotes.datafeed.StockTicker");

        // just parse our args and pass them on
        if (args.length < 4)
        {
            printUsage();
            System.exit(3);
        }
        String propsFile = null;
        String[] servers = new String[args.length - 3];
        long delay = 0;
        int updatesPerBurst = 0;
        try {
            delay = Long.parseLong( args[0] );

            updatesPerBurst = Integer.parseInt( args[1] );
            propsFile = args[2];

            for (int i = 0; i < args.length - 3; i++)
            {
                servers[i] = args[i + 3];
                log.debug("servers[" + i + "]: " + args[i+3]);
            }
        } catch( Exception e ) {
            e.printStackTrace();
            log.error(e.toString(), e);
            printUsage();
        }

        // new up a ticker
        StockTicker ticker = new StockTicker( propsFile, servers  );
        // we should be ready to go now...
        ticker.generateQuotes( delay, updatesPerBurst, -1 );
    }



    /**
      * Constructor with props file and array of object home names.
      *
      * @category Baseline
      * @param propsFile Path to the stockinfo.props file which should contain entries of the form: <br>
      *                  symbol=id:Company Name:inital price  i.e. SONS=1:Sonus Networks:1.45
      * @param servers array of object home names to bind to.
      */
    public StockTicker( String propsFile, String[] servers )
    {
        // set class var
        this.servers = servers;

        //need list of servers here
        tickReceivers = new TickReceiver[servers.length];
        for (int i = 0; i < servers.length; i++)
        {
            this.bindToStockServer(i);
        }

        stockList = new Vector();
        setUp(propsFile);
    }


    /*
     * Sets up the stock ticker.
     *
     * @category Baseline
     * @param propsFile file name of stock ticker properties file
     */
    private void setUp(String propsFile)
    {
        loadStockProps(propsFile);

        StringTokenizer tk = null;
        String symbol = null;
        int id;
        String name = null;
        double initialPrice;

        // parse through our props file and get all of our stock info
        Enumeration propNames = stockProps.propertyNames();

        long now = new Date().getTime();


        log.info( "------------   Sending inital prices   ------------" );
        while( propNames.hasMoreElements() ) {
             symbol = (String)propNames.nextElement();
             String entry = stockProps.getProperty( symbol );

             tk = new StringTokenizer( entry, DELIM );
             id = Integer.parseInt( tk.nextToken() );
             name = tk.nextToken();
             initialPrice = Double.parseDouble( tk.nextToken() );

             // create and add a StockInfo object for each entry
             StockInfo stock = new StockInfo ( id,
                                               symbol,
                                               name,
                                               initialPrice,
                                               now );
             stockList.addElement( stock );
             log.debug( "added stock: " + stock );

             // send each stock's initial info
             sendQuote( stock );
        }

        log.info( "------------   Done sending inital prices   ------------\n" );
    }


    /*
     * Test function.
     *
     * @param delay delay between sending one stock price and another
     * @param updatesPerBurst number of updates to send per burst
     * @param cycles used for testing only use -1 normally
     */
    public void testSend(long delay, int updatesPerBurst, int cycles)
    {
        this.generateQuotes(delay, updatesPerBurst, cycles);
    }


    /**
     * Starts the process of actually sending out updates.
     *
     * cycles is added for testing -- use -1 for inf.
     *
     * @category Baseline
     * @param delay delay between sending one stock price and another
     * @param updatesPerBurst number of updates to send per burst
     * @param cycles used for testing only use -1 normally
     */
    private void generateQuotes( long delay, int updatesPerBurst, int cycles )
    {

        log.info( "------------   Beginning to generate price updates   ------------" );
        int numStocks = stockList.size();

        Random randomGen = new Random();
        int loops = 0;
        while( cycles == -1 || loops < cycles ) {
            loops++;
            for( int i = 0; i < updatesPerBurst; i++ ) {
                // return a (pseudo)random number between 0 and size
                int index = randomGen.nextInt( numStocks );
                log.debug( "index: " + index ) ;

                // get a price changes - no more than $.50
                double change = ((double)randomGen.nextInt( 50 )) / 100;

                // make the price go up or down
                int num = randomGen.nextInt();
                if( num % 2 != 0 ) {
                    change = change * -1;
                }
                log.debug( "change: " + change );

                StockInfo stock = (StockInfo)stockList.elementAt(index);
                double tmpPrice = stock.getPrice() + change;

                BigDecimal bigChange = new BigDecimal( tmpPrice );
                bigChange = bigChange.setScale( 2, BigDecimal.ROUND_HALF_UP );

                stock.setPrice( bigChange.doubleValue() );
                stock.setTimestamp( new Date().getTime() );

                log.info( stock.toString() );

                // actually send the update

                sendQuote( stock );
                try {
                    Thread.currentThread().sleep( delay );
                } 
                    catch( InterruptedException ie ) {  }
            }
        }
    }


    /**
     * sends the quote to all of the servers
     *
     * @category Baseline
     * @param stock stock to send
     */
    private void sendQuote(StockInfo stock)
    {
        for (int i = 0; i < tickReceivers.length; i++)
        {
            // if we think there is a connect to the server, send the stock
            if( tickReceivers[i] != null )
            {
                sendQuoteToServer(stock, i);
                log.info("Sending quote to server: " + (i+1));
            }
        }
    }


    /**
     * This function was added so that the sendQuote function will
     * send quotes to all servers regardless of which servers are up or
     * down.
     *
     * This is a real kludge - there is probably a better way.
     *
     * @category Baseline, Fault Tolerance
     * @param stock stock to send
     * @param i
     */
    private void sendQuoteToServer(StockInfo stock, int i)
    {
        try
        {
            tickReceivers[i].updateStockInfo(stock);
        }
        catch( RemoteException re )
        {
            log.error("Caught a remote exception: " + re);
            log.info("Unable to send quote to server: " + i + ".\n Trying to reconnect to server.");
            tickReceivers[i] = null;
            this.bindToStockServer(i);
        }
        //AN - added b/c I got a java.lang.IllegalStateException that I didn't
        // know we needed to catch  ** we should probably rethink this a little**
        catch (Exception e)
        {
            log.error("Caught an unexpected exception: " + e);
            log.info("Unable to send quote to server: " + i + ".\n Trying to reconnect to server.");
            tickReceivers[i] = null;
            this.bindToStockServer(i);
        }
    }


    /*
     * Binds to the ith server.  This function assumes that the home name array and
     * server array are in the same order.
     *
     * @category Baseline, Fault Tolerance
     * @param i index into server array of the server to bind to
     */
    private void bindToStockServer(int i)
    {
        // create a final var that can be used in thread
        final int num = i;

        // create new thread to bind to server
        Thread t = new Thread()
        {
            public void run()
            {
                // display logging info
                log.info("binding to " + servers[num]);

                TickReceiverHome tickReceiverHome = null;
                while(tickReceiverHome == null)
                {
                    try
                    {
                        Context initial = new InitialContext();

                        Object objref = initial.lookup(servers[num]);
                        tickReceiverHome = (TickReceiverHome) PortableRemoteObject.narrow(objref, TickReceiverHome .class);

                        // try connecting to the server
                        tickReceivers[num]  = tickReceiverHome.create();
                    }
                    catch(NamingException ne)
                    {
                        log.error("Could not get the Home interface for TickReceiver: " + ne);
                        log.error("Server: " + (num + 1) + " may not be up.  Trying again in 5 seconds...");
                    }
                    // AN - moved catches here
                    // we need to also try again if the home interface is obtained (the server is deployed), 
                    // but the server is not yet running
                    catch( CreateException ce )
                    {
                        log.error( "Could not create TickReceieverSessionBean: " + ce );
                        log.error("Server: " + (num + 1) + " may not be up.  Trying again in 5 seconds...");
                        tickReceiverHome = null;
                    }
                    catch( RemoteException re )
                    {
                        log.error( "Could not create TickReceieverSessionBean: " + re );
                        log.error("Server: " + (num + 1) + " may not be up.  Trying again in 5 seconds...");
                        tickReceiverHome = null;
                    }
                    // AN - added this here - b/c I think we at least need to do something if an exception
                    // occurs  - do we want to kill it, restart server ??
                    catch(Exception e)
                    {
                        log.error( "Unexpected exception occured when binding to stock server: " + e );
                        log.error("Server: " + (num + 1) + " may not be up.  Trying again in 5 seconds...");
                        tickReceiverHome = null;                    
                    }
                    
                    // if unable to connect, sleep and then try again
                    if(tickReceiverHome == null)
                    {
                        try
                        {
                            Thread.currentThread().sleep(5000);
                        }
                        catch( InterruptedException ie ) {}
                    }
                }
                log.info("Bound to server: " + (num+1));
            }
        };

        t.start();
    }


    /*
     * Loads the properties from the stock ticker properties file.
     *
     * @category Baseline
     * @param propsFile name of stock ticker properties file
     */
    private void loadStockProps( String propsFile )
    {
        stockProps = new Properties();
        try {
            stockProps.load( new FileInputStream( propsFile ) );
        } catch( IOException ioe ) {
            log.error( "Could not load stock information properties file: " + propsFile );
            log.error( "Exiting..." );
            System.exit(1);
        }
    }


    /**
     * Prints the usage string and exits.
     *
     * @category Baseline
     */
    private static void printUsage() {

        log.fatal( "Usage: java powerquotes.datafeed.StockTicker delay numUpdates propsFile server0 [server1] ..." );
        log.fatal( "  delay: is the time in milliseconds to wait between updates" );
        log.fatal( "  numUpdates: number of stock updates to send out per cycle" );
        log.fatal( "  propFile: path to stockinfo.props" );
        System.exit(0);
    }

}

