//------------------------------------------------------------------------------------
/**
 * <p>Title: ClientUpdateSessionBean </p>
 * <b> Category: </b> Baseline
 * <b> Behavior:</b> Provides a way for a client to get stock information.
 * <b> Inherits from:</b> javax.ejb.SessionBean
 * <b> Clients:</b> PowerQuotesClient
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//------------------------------------------------------------------------------------
package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import java.util.*;
import powerquotes.ejb.*;
import powerquotes.common.*;
import org.apache.log4j.Logger;

/** 
 * Provides a way for a client to get stock information.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 *
 * @ejb:bean name="ClientUpdate"
 *           display-name="Client Update Session Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/powerquotes/ClientUpdate"
 *           view-type="remote"
 * @ejb:home remote-package="powerquotes.ejb"
 * @ejb:interface remote-package="powerquotes.ejb"
 * @ejb:util generate="physical"
*/
public class ClientUpdateSessionBean implements SessionBean {

    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.ClientUpdateSessionBean");

    
    //================================================================================
    // Client Interface Methods
    //================================================================================
    /** This method will subscribe the given user for the given stocks.
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param userName name of user subscribing
     * @param stockId array of stocks to subscribe to
     * @throws ServerNotReadyException thrown if server is not ready to service client operations
     *
     * @ejb:interface-method view-type="remote"
     */
    public void subscribe(String userName, int[] stockId) throws ServerNotReadyException
    {

        // print status
        log.debug("trying to subscribe for " + userName);

        // first find the cache object
        StockCache cache = StockCache.getInstance();

        if( cache.isInitialized() ) {
        
            // first find the subscription object
            Subscription subs = Subscription.getInstance();

            // update the subscription's view of the user
            subs.setSubscription(userName, stockId);
            
            // print that client has subscribed
            log.info("Client has subscribed to receive stock updates for: " + userName);
        }
        else {
            throw new ServerNotReadyException();
        }
    }


    /** This method will return the list of all stocks.
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @return a vector containing all stocks available
     * @throws ServerNotReadyException thrown if server is not ready to service client operations
     *
     * @ejb:interface-method view-type="remote"
     */
    public Vector getAllStocks() throws ServerNotReadyException
    {

        // print status
        log.debug("get all stocks");
        // first find the cache object
        StockCache cache = StockCache.getInstance();
        if( cache.isInitialized() ) {
            // return all stocks from cache
            return(new Vector(cache.getAllStocks()));
        }
        throw new ServerNotReadyException();
    }


    /** This method returns the list of stocks that the user is interested in
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param userName the name ofthe user interested in an update
     * @return a vector containing all stocks the user is currently subsribed for
     * @throws ServerNotReadyException thrown if server is not ready to service client operations
     *
     * @ejb:interface-method view-type="remote"
     */
    public Vector getUpdatedStocks(String userName) throws ServerNotReadyException 
    {
        
        // print status
        log.debug("get updated stocks");
        StockCache cache = StockCache.getInstance();
        
        if( cache.isInitialized() ) {
            // create vector
            Vector stocks = new Vector();
            
            // first find the subscription & cache objects
            Subscription subs = Subscription.getInstance();
            
            // get stocks the user is interested in
            int[] stockId = subs.getSubscription(userName);
            
            // get the current value for all stocks
            for(int i=0; i< stockId.length; i++)
                {
                    stocks.add(cache.getStock(new Integer(stockId[i])));
                }
            
            return stocks;
        }
        throw new ServerNotReadyException();
    }



    //================================================================================
    // EJB Methods
    //================================================================================
    /** Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method view-type="remote"
     */
    public void ejbCreate() throws CreateException {
        log.debug("ejbCreate()");
    }
    public void setSessionContext(SessionContext sc) {
        log.debug("setSessionContext()");
    }
    public void ejbRemove() {
        log.debug("ejbRemove()");
    }
    public void ejbActivate() {
        log.debug("ejbActivate()");
    }
    public void ejbPassivate() {
        log.debug("ejbPassivate()");
    }

}
