//------------------------------------------------------------------------------------
/**
 * <p>Title: UserProfileSessionBean </p>
 * <b> Category:  </b> Baseline
 * <b> Behavior: </b> Provides an interface to the client for getting and setting a user profile.
  * <b> Inherits from: </b> javax.ejb.SessionBean
 * <b> Clients: </b> PowerQuotesClient
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 */
//------------------------------------------------------------------------------------
package powerquotes.ejb.impl;

// Import
import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import powerquotes.ejb.*;
import powerquotes.common.*;
import org.apache.log4j.Logger;
import java.io.NotSerializableException;


/** 
 * Provides an interface to the client for getting and setting a user profile.
 *
 * @author Dan Abramovich, Justin Kenlon, April Navarro, Kevin Steppe
 * @version 1.0
 *
 * @ejb:bean name="UPSession"
 *           display-name="UserProfile Session Bean"
 *           type="Stateless"
 *           transaction-type="Container"
 *           jndi-name="ejb/powerquotes/UPSession"
 *           view-type="remote"
 * @ejb:home remote-package="powerquotes.ejb"
 * @ejb:interface remote-package="powerquotes.ejb"
*/
public class UserProfileSessionBean implements SessionBean {

    protected static Logger log = Logger.getLogger("powerquotes.ejb.impl.UserProfileSessionBean");                

    
    //================================================================================
    // Client Interface Methods
    //================================================================================
    /** This method returns the profile that is associated with the given user name.
     *  Our performance requirements require a return within 20 seconds --
     *  this will include a set of current stock prices.
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param userName the user name of the client
     * @return the client's profile associated with the given user name
     * @throws NoSuchUserException if the user cannot be found in the database
     * @throws StockServerException if there is a problem communicating with any other beans
     * @throws NotSerializableException if the UserProfile cannot be returned 
     * @ejb:interface-method view-type="remote"
     */
    public UserProfile getUserProfile(String userName)
        throws NoSuchUserException, StockServerException, NotSerializableException
    {
        // display status
        log.debug("UserProfileSessionBean: getUserProfile for " + userName);        

        UserProfileBeanUtil beanUtil = null;
        UserProfileBeanLocalHome home = null;
        UserProfileBeanLocal theBean = null;   
        
        // try to get the user profile from the UserProfileBean
        UserProfile up=null;
        try 
        {
            // get the bean
            home = (UserProfileBeanLocalHome) beanUtil.getLocalHome();        
            theBean = home.findByUserName(userName);
            
            // return the information in a UserProfile object
            up = new UserProfile();
            up.setUserName(theBean.getUserName());
            up.setUserId(theBean.getUserId());
            up.setProfileName(theBean.getProfileName());

            up.setProfileId(theBean.getProfileId().intValue());
            up.setStockList(theBean.getStockList());
        }
        catch(javax.naming.NamingException ne) 
        {
            // problem with server - unable to get local home interface
            log.error("Caught a naming exception! " + ne.toString(), ne);
            throw new StockServerException();
        }
        catch(javax.ejb.ObjectNotFoundException onfe) 
        {
            // unable to find user name in database
           log.error("Caught an object not found exception! " + onfe.toString(), onfe);
           throw new NoSuchUserException();
        } 
        // any other exceptions - problem with server
        catch(Exception e)
        {
            // problem with server - unexpteced exception
            log.error("Caught an unexpected exception! " + e.toString(), e);
            throw new StockServerException();
        }   
        
        return(up);        
    }


    /** This method persists the profile to persistant storage.
     *  If no profile for this user exists in the DB it is created
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param profile The profile to set or create.
     * @throws StockServerException if there is a problem communicating with any other beans
     *
     * @ejb:interface-method view-type="remote"
     */
    public void setUserProfile(UserProfile profile)
        throws StockServerException      
    {
        // display status
        log.debug("UserProfileSessionBean: setUserProfile for " + profile.getUserName());

        UserProfileBeanUtil beanUtil = null;
        try 
        {
            UserProfileBeanLocalHome home = (UserProfileBeanLocalHome) beanUtil.getLocalHome();
            UserProfileBeanLocal theBean = null;
            
            // try to set profile, if it does not exist then create a new one
            try 
            {
                theBean = (UserProfileBeanLocal) home.findByUserName(profile.getUserName());
            } 
            catch(javax.ejb.ObjectNotFoundException onfe) 
            {
                log.debug( "No profile for this user - going to create...");
                theBean = home.create(null);
            }
            
            theBean.setUserName(profile.getUserName());
            theBean.setUserId(profile.getUserId());
            theBean.setProfileName(profile.getProfileName());            
            theBean.setStockList(profile.getStockList());
        }
        catch(javax.naming.NamingException ne) 
        {
            // problem with server - unable to get local home interface
            log.error("Caught a naming exception! " + ne.toString(), ne);
            throw new StockServerException();
        }
        catch(javax.ejb.CreateException ce)
        {
            // problem with server - unable to create the bean 
            log.error("Caught a create exception! " + ce.toString(), ce);
            throw new StockServerException();        
        }
        // any other exceptions - problem with server
        catch(Exception e)
        {
            // problem with server - unexpteced exception
            log.error("Caught an unexpected exception! " + e.toString(), e);
            throw new StockServerException();
        }   
    }

    /** This method creates a new user for the system.
     *
     * @category Baseline
     * @clients PowerQuotesClient
     * @param userName The name of the user to be created.
     * @ejb:interface-method view-type="remote"
     */
    public void createUser(String userName) 
    {
        log.debug("UserProfileSessionBean: createUser for " + userName);
    }

    
    //================================================================================
    // EJB Methods
    //================================================================================

    /** Create the Session Bean.
     *
     * @throws CreateException
     *
     * @ejb:create-method view-type="remote"
     */
    public void ejbCreate() throws CreateException
    {
        log.debug("UserProfileSessionBean: ejbCreate()");
    }
    public void setSessionContext(SessionContext sc)
    {
        log.debug("UserProfileSessionBean: setSessionContext()");
    }
    public void ejbRemove() 
    {
        log.debug("UserProfileSessionBean: ejbRemove()");
    }
    public void ejbActivate() 
    {
        log.debug("UserProfileSessionBean: ejbActivate()");
    }
    public void ejbPassivate() 
    {
        log.debug("UserProfileSessionBean: ejbPassivate()");
    }
}
